@@ -3,4 +3,4 @@ SRC_PATH = [
     "/Users/osso/src/inspire",
 ]
 
-USE_CONSOLE = True
+USE_CONSOLE = False
