@@ -31,7 +31,9 @@ DIRS = {
     'py': 'lib/python/invenio',
     'js': 'var/www/js',
     'css': 'var/www/css',
-    'conf': None,
+    'conf': 'etc',
+    'bft': 'etc/bibformat/format_templates',
+    'bfo': 'etc/bibformat/output_formats',
 }
 
 STATIC_FILES = {
