@@ -6,6 +6,7 @@ import optparse
 import socket
 import traceback
 import logging
+import re
 from itertools import chain
 
 try:
@@ -31,8 +32,11 @@ except NameError:
     # Python 2.4 compatibility
     BaseException = Exception
 
+
 DESCRIPTION = "Invenio web server for development"
 USAGE_MESSAGE = "python serve.py [-bp]"
+RE_BIBFORMAT_ELEMENTS = re.compile(u"^(/modules/bibformat/lib/elements/.+\.py|/bibformat/format_elements/.+\.py)",
+                                   re.U|re.I)
 
 
 def get_extension(filename):
@@ -53,24 +57,35 @@ def generate_invenio_files_list(invenio_path=config.SRC_PATH):
         invenio_path = [invenio_path]
 
     def iter_files(dirname, filenames):
-        return (os.path.join(dirname, filename) \
+        return (os.path.join(dirname, filename)
                       for filename in filenames if valid_extension(filename))
 
     def iter_folder(folder):
         return chain(*(
-            iter_files(dirname, filenames) \
+            iter_files(dirname, filenames)
                                 for dirname, _, filenames in os.walk(folder)
         ))
 
     return chain(chain(*(iter_folder(folder) for folder in invenio_path)),
-                 ['%setc/invenio-local.conf' % config.INSTALL_PATH])
+                [os.path.join(config.INSTALL_PATH, 'etc/invenio-local.conf')])
 
 
 def select_destination_path(filename, install_path=config.INSTALL_PATH,
                                                             dests=config.DIRS):
     dest = None
-    lib_dir = dests[get_extension(filename)]
+
+    rel_path = filename
+    for src_path in config.SRC_PATH:
+        if rel_path.startswith(src_path):
+            rel_path = rel_path.replace(src_path, '')
+            break
+
+    ext = get_extension(filename)
+    lib_dir = dests[ext]
     if lib_dir:
+        if ext == 'py':
+            if RE_BIBFORMAT_ELEMENTS.match(rel_path):
+                lib_dir = os.path.join(lib_dir, 'bibformat_elements')
         dest = os.path.join(install_path,
                             lib_dir,
                             os.path.basename(filename))
@@ -78,8 +93,8 @@ def select_destination_path(filename, install_path=config.INSTALL_PATH,
 
 
 def update_conf():
-    _log('info', 'updating configuration')
-    ret = os.system('%s/bin/inveniocfg --update-config-py' \
+    _log('info', ' * Updating configuration')
+    ret = os.system('%s/bin/inveniocfg --update-config-py'
                                                          % config.INSTALL_PATH)
     if ret == 0:
         _log('info', 'done updating')
@@ -93,6 +108,7 @@ def reloader_loop(files, reloader=None, interval=1):
     mtimes = {}
     while 1:
         has_changes = False
+        has_conf_changes = False
 
         for filename in set(files):
             try:
@@ -113,22 +129,27 @@ def reloader_loop(files, reloader=None, interval=1):
                 # finish writing the file
                 time.sleep(0.1)
 
-                if os.path.basename(filename) in (config.CONFIG_FILENAME,
-                                                 config.LOCAL_CONFIG_FILENAME):
-                    update_conf()
-
                 if os.path.basename(filename) == config.LOCAL_CONFIG_FILENAME:
                     dest = None
                 else:
                     dest = select_destination_path(filename)
+
                 if dest:
-                    _log('info', ' * Detected change in %r, '\
+                    _log('info', ' * Detected change in %r, '
                                         'copying to %s' % (filename, dest))
                     shutil.copyfile(filename, dest)
                 else:
                     _log('info', ' * Detected change in %r' % filename)
+
+                if os.path.basename(filename) in (config.CONFIG_FILENAME,
+                                                 config.LOCAL_CONFIG_FILENAME):
+                    has_conf_changes = True
+
                 has_changes = True
 
+        if has_conf_changes:
+            update_conf()
+
         if has_changes and reloader:
             reloader()
             if not reloader.worker_pid:
@@ -200,7 +221,7 @@ def start_server(options, server_socket, static_files=config.STATIC_FILES):
 
     def timed_out_request(f, *args, **kwargs):
         def handler(signum, frame):
-            _log('info', "\033[01;31mYour request took more than %s" \
+            _log('info', "\033[01;31mYour request took more than %s"
                          " seconds to process\033[0m" % config.REQUEST_TIMEOUT)
         signal.signal(signal.SIGALRM, handler)
         signal.alarm(config.REQUEST_TIMEOUT)
@@ -244,13 +265,13 @@ def parse_cli_options():
     parser.add_option('-p', dest='bind_port', type='int', default=4000,
                                     help='Port to bind to')
     parser.add_option('--no-reload', action='store_false', dest='auto_reload',
-                      default=True, help='Disable automatic reloading\n'\
+                      default=True, help='Disable automatic reloading\n'
                       'when a source file is changed')
     parser.add_option('--no-http', action='store_false', dest='serve_http',
-                      default=True, help='Disable http server, only update ' \
+                      default=True, help='Disable http server, only update '
                       'invenio install')
     parser.add_option('--pdb', dest='use_pdb', default=False,
-                      action='store_true', help='Drop to python debugger\n' \
+                      action='store_true', help='Drop to python debugger\n'
                       'on errors')
     parser.add_option('-s', action='append', dest='src_path', metavar='SRC_PATH',
                       default=[], help='Source folder (one or more)')
